@@ -601,6 +601,7 @@ struct AnyUnsafe : public ObjectUnsafe {
 
 /*! \brief String-aware Any equal functor */
 struct AnyHash {
+ public:
   /*!
    * \brief Calculate the hash code of an Any
    * \param a The given Any
@@ -623,13 +624,79 @@ struct AnyHash {
       return details::StableHashCombine(src.data_.type_index,
                                         details::StableHashBytes(src_str->data, src_str->size));
     } else {
+      if (src.data_.type_index >= TypeIndex::kTVMFFIStaticObjectBegin) {
+        static const TVMFFITypeAttrColumn* custom_hash_column = GetAnyHashTypeAttrColumn();
+        if (custom_hash_column != nullptr) {
+          int32_t offset = src.data_.type_index - custom_hash_column->begin_index;
+          if (offset >= 0 && offset < custom_hash_column->size) {
+            const TVMFFIAny& custom_any_hash = custom_hash_column->data[offset];
+            if (custom_any_hash.type_index != TypeIndex::kTVMFFINone) {
+              return details::StableHashCombine(src.data_.type_index,
+                                                CallCustomAnyHash(custom_any_hash, src));
+            }
+          }
+        }
+      }
       return details::StableHashCombine(src.data_.type_index, src.data_.v_uint64);
     }
   }
+
+ private:
+  /*!
+   * \brief Get the type attribute column for any hash.
+   * \return The type attribute column for any hash.
+   */
+  static const TVMFFITypeAttrColumn* GetAnyHashTypeAttrColumn() {
+    constexpr const char* kAttrName = "__any_hash__";
+    TVMFFIByteArray attr_name =
+        TVMFFIByteArray{kAttrName, std::char_traits<char>::length(kAttrName)};
+    return TVMFFIGetTypeAttrColumn(&attr_name);
+  }
+
+  /*!
+   * \brief Call the custom any hash function registered in type attribute column.
+   * \param custom_any_hash The custom any hash function object or function pointer.
+   * \param src The source Any.
+   * \return The hash value.
+   */
+  static uint64_t CallCustomAnyHash(const TVMFFIAny& custom_any_hash, const Any& src) {
+    // NOTE: we explicitly use low-level ABI here since we do not want to have dep on function.h
+    // it also keeps the logic simple
+    if (custom_any_hash.type_index == TypeIndex::kTVMFFIOpaquePtr) {
+      // we allow this attribute to be a function pointer for fast path
+      using FCustomAnyHashPtr = int64_t (*)(const Any&);
+      FCustomAnyHashPtr hash_func = reinterpret_cast<FCustomAnyHashPtr>(custom_any_hash.v_ptr);
+      TVM_FFI_ICHECK_NOTNULL(hash_func);
+      return static_cast<uint64_t>(
+          (*hash_func)(src));  // NOLINT(clang-analyzer-core.CallAndMessage)
+    } else {
+      // alternatively it can be a ffi.Function object.
+      TVM_FFI_ICHECK_EQ(custom_any_hash.type_index, TypeIndex::kTVMFFIFunction);
+      TVMFFIAny arg = src.data_;
+      TVMFFIAny result;
+      result.type_index = TypeIndex::kTVMFFINone;
+      result.zero_padding = 0;
+      result.v_int64 = 0;
+      TVMFFIFunctionCell* func_cell = TVMFFIFunctionGetCellPtr(custom_any_hash.v_obj);
+      if (func_cell->cpp_call != nullptr) {
+        // Fast path: invoke C++ ABI call directly when available.
+        using FCppCall = void (*)(const void*, const TVMFFIAny*, int32_t, TVMFFIAny*);
+        reinterpret_cast<FCppCall>(func_cell->cpp_call)(custom_any_hash.v_obj, &arg, 1, &result);
+      } else {
+        if (func_cell->safe_call(custom_any_hash.v_obj, &arg, 1, &result) != 0) {
+          throw details::MoveFromSafeCallRaised();
+        }
+      }
+      Any result_any = details::AnyUnsafe::MoveTVMFFIAnyToAny(&result);
+      TVM_FFI_ICHECK_EQ(result_any.type_index(), TypeIndex::kTVMFFIInt);
+      return static_cast<uint64_t>(result_any.data_.v_int64);
+    }
+  }
 };
 
 /*! \brief String-aware Any hash functor */
 struct AnyEqual {
+ public:
   /*!
    * \brief Check if the two Any are equal
    * \param lhs left operand.
@@ -657,6 +724,18 @@ struct AnyEqual {
             details::AnyUnsafe::CopyFromAnyViewAfterCheck<const details::BytesObjBase*>(rhs);
         return Bytes::memequal(lhs_str->data, rhs_str->data, lhs_str->size, rhs_str->size);
       }
+      if (lhs.data_.type_index >= TypeIndex::kTVMFFIStaticObjectBegin) {
+        static const TVMFFITypeAttrColumn* custom_equal_column = GetAnyEqualTypeAttrColumn();
+        if (custom_equal_column != nullptr) {
+          int32_t offset = lhs.data_.type_index - custom_equal_column->begin_index;
+          if (offset >= 0 && offset < custom_equal_column->size) {
+            const TVMFFIAny& custom_any_equal = custom_equal_column->data[offset];
+            if (custom_any_equal.type_index != TypeIndex::kTVMFFINone) {
+              return CallCustomAnyEqual(custom_any_equal, lhs, rhs);
+            }
+          }
+        }
+      }
       return false;
     } else {
       // type_index mismatch, if index is not string, return false
@@ -692,6 +771,59 @@ struct AnyEqual {
       return false;
     }
   }
+
+ private:
+  /*!
+   * \brief Get the type attribute column for any equal.
+   * \return The type attribute column for any equal.
+   */
+  static const TVMFFITypeAttrColumn* GetAnyEqualTypeAttrColumn() {
+    constexpr const char* kAttrName = "__any_equal__";
+    TVMFFIByteArray attr_name =
+        TVMFFIByteArray{kAttrName, std::char_traits<char>::length(kAttrName)};
+    return TVMFFIGetTypeAttrColumn(&attr_name);
+  }
+
+  /*!
+   * \brief Call the custom any equal function registered in type attribute column.
+   * \param custom_any_equal The custom any equal function object or function pointer.
+   * \param lhs The left-hand side Any.
+   * \param rhs The right-hand side Any.
+   * \return The equality result.
+   */
+  static bool CallCustomAnyEqual(const TVMFFIAny& custom_any_equal, const Any& lhs,
+                                 const Any& rhs) {
+    // NOTE: we explicitly use low-level ABI here since we do not want to have dep on function.h
+    // it also keeps the logic simple
+    if (custom_any_equal.type_index == TypeIndex::kTVMFFIOpaquePtr) {
+      // we allow this attribute to be a function pointer for fast path
+      using FCustomAnyEqualPtr = bool (*)(const Any&, const Any&);
+      FCustomAnyEqualPtr equal_func = reinterpret_cast<FCustomAnyEqualPtr>(custom_any_equal.v_ptr);
+      TVM_FFI_ICHECK_NOTNULL(equal_func);
+      return (*equal_func)(lhs, rhs);  // NOLINT(clang-analyzer-core.CallAndMessage)
+    } else {
+      // alternatively it can be a ffi.Function object.
+      TVM_FFI_ICHECK_EQ(custom_any_equal.type_index, TypeIndex::kTVMFFIFunction);
+      TVMFFIAny args[2] = {lhs.data_, rhs.data_};
+      TVMFFIAny result;
+      result.type_index = TypeIndex::kTVMFFINone;
+      result.zero_padding = 0;
+      result.v_int64 = 0;
+      TVMFFIFunctionCell* func_cell = TVMFFIFunctionGetCellPtr(custom_any_equal.v_obj);
+      if (func_cell->cpp_call != nullptr) {
+        // Fast path: invoke C++ ABI call directly when available.
+        using FCppCall = void (*)(const void*, const TVMFFIAny*, int32_t, TVMFFIAny*);
+        reinterpret_cast<FCppCall>(func_cell->cpp_call)(custom_any_equal.v_obj, args, 2, &result);
+      } else {
+        if (func_cell->safe_call(custom_any_equal.v_obj, args, 2, &result) != 0) {
+          throw details::MoveFromSafeCallRaised();
+        }
+      }
+      Any result_any = details::AnyUnsafe::MoveTVMFFIAnyToAny(&result);
+      TVM_FFI_ICHECK(result_any.type_index() == TypeIndex::kTVMFFIBool);
+      return result_any.data_.v_int64 != 0;
+    }
+  }
 };
 }  // namespace ffi
 
