@@ -22,6 +22,8 @@
  * \brief Structural equal implementation.
  */
 #include <tvm/ffi/container/array.h>
+#include <tvm/ffi/container/dict.h>
+#include <tvm/ffi/container/list.h>
 #include <tvm/ffi/container/map.h>
 #include <tvm/ffi/container/shape.h>
 #include <tvm/ffi/container/tensor.h>
@@ -31,6 +33,7 @@
 
 #include <cmath>
 #include <unordered_map>
+#include <utility>
 
 namespace tvm {
 namespace ffi {
@@ -103,10 +106,18 @@ class StructEqualHandler {
         return CompareArray(AnyUnsafe::MoveFromAnyAfterCheck<Array<Any>>(std::move(lhs)),
                             AnyUnsafe::MoveFromAnyAfterCheck<Array<Any>>(std::move(rhs)));
       }
+      case TypeIndex::kTVMFFIList: {
+        return CompareList(AnyUnsafe::MoveFromAnyAfterCheck<List<Any>>(std::move(lhs)),
+                           AnyUnsafe::MoveFromAnyAfterCheck<List<Any>>(std::move(rhs)));
+      }
       case TypeIndex::kTVMFFIMap: {
         return CompareMap(AnyUnsafe::MoveFromAnyAfterCheck<Map<Any, Any>>(std::move(lhs)),
                           AnyUnsafe::MoveFromAnyAfterCheck<Map<Any, Any>>(std::move(rhs)));
       }
+      case TypeIndex::kTVMFFIDict: {
+        return CompareMap(AnyUnsafe::MoveFromAnyAfterCheck<Dict<Any, Any>>(std::move(lhs)),
+                          AnyUnsafe::MoveFromAnyAfterCheck<Dict<Any, Any>>(std::move(rhs)));
+      }
       case TypeIndex::kTVMFFIShape: {
         return CompareShape(AnyUnsafe::MoveFromAnyAfterCheck<Shape>(std::move(lhs)),
                             AnyUnsafe::MoveFromAnyAfterCheck<Shape>(std::move(rhs)));
@@ -255,8 +266,8 @@ class StructEqualHandler {
     }
   }
 
-  // NOLINTNEXTLINE(performance-unnecessary-value-param)
-  bool CompareMap(Map<Any, Any> lhs, Map<Any, Any> rhs) {
+  template <typename MapType>
+  bool CompareMap(const MapType& lhs, const MapType& rhs) {
     if (lhs.size() != rhs.size()) {
       // size mismatch, and there is no path tracing
       // return false since we don't need informative error message
@@ -302,6 +313,17 @@ class StructEqualHandler {
 
   // NOLINTNEXTLINE(performance-unnecessary-value-param)
   bool CompareArray(ffi::Array<Any> lhs, ffi::Array<Any> rhs) {
+    return CompareSequence(std::move(lhs), std::move(rhs));
+  }
+
+  // NOLINTNEXTLINE(performance-unnecessary-value-param)
+  bool CompareList(ffi::List<Any> lhs, ffi::List<Any> rhs) {
+    return CompareSequence(std::move(lhs), std::move(rhs));
+  }
+
+  template <typename SeqType>
+  // NOLINTNEXTLINE(performance-unnecessary-value-param)
+  bool CompareSequence(SeqType lhs, SeqType rhs) {
     if (lhs.size() != rhs.size()) {
       // fast path, size mismatch, and there is no path tracing
       // return false since we don't need informative error message
@@ -437,7 +459,9 @@ Optional<reflection::AccessPathPair> StructuralEqual::GetFirstMismatch(const Any
 
 TVM_FFI_STATIC_INIT_BLOCK() {
   namespace refl = tvm::ffi::reflection;
-  refl::GlobalDef().def("ffi.GetFirstStructuralMismatch", StructuralEqual::GetFirstMismatch);
+  refl::GlobalDef()
+      .def("ffi.StructuralEqual", StructuralEqual::Equal)
+      .def("ffi.GetFirstStructuralMismatch", StructuralEqual::GetFirstMismatch);
   // ensure the type attribute column is presented in the system even if it is empty.
   refl::EnsureTypeAttrColumn("__s_equal__");
 }
