@@ -22,6 +22,8 @@
  * \brief Structural equal implementation.
  */
 #include <tvm/ffi/container/array.h>
+#include <tvm/ffi/container/dict.h>
+#include <tvm/ffi/container/list.h>
 #include <tvm/ffi/container/map.h>
 #include <tvm/ffi/container/shape.h>
 #include <tvm/ffi/container/tensor.h>
@@ -78,9 +80,16 @@ class StructuralHashHandler {
       case TypeIndex::kTVMFFIArray: {
         return HashArray(AnyUnsafe::MoveFromAnyAfterCheck<Array<Any>>(std::move(src)));
       }
+      case TypeIndex::kTVMFFIList: {
+        return HashList(AnyUnsafe::MoveFromAnyAfterCheck<List<Any>>(std::move(src)));
+      }
       case TypeIndex::kTVMFFIMap: {
         return HashMap(AnyUnsafe::MoveFromAnyAfterCheck<Map<Any, Any>>(std::move(src)));
       }
+      case TypeIndex::kTVMFFIDict: {
+        Dict<Any, Any> dict = AnyUnsafe::MoveFromAnyAfterCheck<Dict<Any, Any>>(std::move(src));
+        return HashMapBase(static_cast<const MapBaseObj*>(dict.get()));
+      }
       case TypeIndex::kTVMFFIShape: {
         return HashShape(AnyUnsafe::MoveFromAnyAfterCheck<Shape>(std::move(src)));
       }
@@ -185,9 +194,16 @@ class StructuralHashHandler {
   }
 
   // NOLINTNEXTLINE(performance-unnecessary-value-param)
-  uint64_t HashArray(Array<Any> arr) {
-    uint64_t hash_value = details::StableHashCombine(arr->GetTypeKeyHash(), arr.size());
-    for (const auto& elem : arr) {
+  uint64_t HashArray(Array<Any> arr) { return HashSequence(std::move(arr)); }
+
+  // NOLINTNEXTLINE(performance-unnecessary-value-param)
+  uint64_t HashList(List<Any> list) { return HashSequence(std::move(list)); }
+
+  template <typename SeqType>
+  // NOLINTNEXTLINE(performance-unnecessary-value-param)
+  uint64_t HashSequence(SeqType seq) {
+    uint64_t hash_value = details::StableHashCombine(seq->GetTypeKeyHash(), seq.size());
+    for (const auto& elem : seq) {
       hash_value = details::StableHashCombine(hash_value, HashAny(elem));
     }
     return hash_value;
@@ -232,10 +248,14 @@ class StructuralHashHandler {
 
   // NOLINTNEXTLINE(performance-unnecessary-value-param)
   uint64_t HashMap(Map<Any, Any> map) {
+    return HashMapBase(static_cast<const MapBaseObj*>(map.get()));
+  }
+
+  uint64_t HashMapBase(const MapBaseObj* map) {
     // Compute a deterministic hash value for the map.
-    uint64_t hash_value = details::StableHashCombine(map->GetTypeKeyHash(), map.size());
+    uint64_t hash_value = details::StableHashCombine(map->GetTypeKeyHash(), map->size());
     std::vector<std::pair<uint64_t, Any>> items;
-    for (auto [key, value] : map) {
+    for (const auto& [key, value] : *map) {
       // if we cannot find order independent hash, we skip the key
       if (auto hash_key = FindOrderIndependentHash(key)) {
         items.emplace_back(*hash_key, value);
