@@ -153,15 +153,6 @@ def test_pyfunc_convert() -> None:
     assert fapply(add, 1, 3.3) == 4.3
 
 
-def test_pyfunc_init() -> None:
-    def add(a: int, b: int) -> int:
-        return a + b
-
-    fadd = tvm_ffi.Function(add)
-    assert isinstance(fadd, tvm_ffi.Function)
-    assert fadd(1, 2) == 3
-
-
 def test_global_func() -> None:
     @tvm_ffi.register_global_func("mytest.echo")
     def echo(x: Any) -> Any:
@@ -293,7 +284,7 @@ def test_function_subclass() -> None:
         # When subclassing a Cython cdef class and overriding `__init__`,
         # special methods like `__call__` may not be inherited automatically.
         # This explicit assignment ensures the subclass remains callable.
-        __call__ = tvm_ffi.Function.__call__  # type: ignore
+        __call__ = tvm_ffi.Function.__call__
 
     f = tvm_ffi.convert(lambda x: x)
     assert isinstance(f, tvm_ffi.Function)
